
#include "xc.h"

#include <pic18f27k42.h>

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <limits.h>
#include <string.h>
#include "i2c1_master.h"
#include "i2c1_master_example.h"


// CONFIG1L
#pragma config FEXTOSC = OFF    // External Oscillator Selection->Oscillator not enabled
#pragma config RSTOSC = HFINTOSC_64MHZ    // Reset Oscillator Selection->HFINTOSC with HFFRQ = 64 MHz and CDIV = 1:1
// CONFIG1H
#pragma config CLKOUTEN = OFF    // Clock out Enable bit->CLKOUT function is disabled
#pragma config PR1WAY = OFF    // PRLOCKED One-Way Set Enable bit->PRLOCK bit can be set and cleared repeatedly
#pragma config CSWEN = OFF    // Clock Switch Enable bit->The NOSC and NDIV bits cannot be changed by user software
#pragma config FCMEN = OFF    // Fail-Safe Clock Monitor Enable bit->Fail-Safe Clock Monitor disabled
// CONFIG2L
#pragma config MCLRE = EXTMCLR    // MCLR Enable bit->If LVP = 0, MCLR pin is MCLR; If LVP = 1, RE3 pin function is MCLR 
#pragma config PWRTS = PWRT_OFF    // Power-up timer selection bits->PWRT is disabled
#pragma config MVECEN = ON   // Multi-vector enable bit->Interrupt contoller does not use vector table to prioritze interrupts
#pragma config IVT1WAY = ON    // IVTLOCK bit One-way set enable bit->IVTLOCK bit can be cleared and set repeatedly
#pragma config LPBOREN = OFF    // Low Power BOR Enable bit->ULPBOR disabled
#pragma config BOREN = SBORDIS    // Brown-out Reset Enable bits->Brown-out Reset enabled , SBOREN bit is ignored
// CONFIG2H
#pragma config BORV = VBOR_2P45    // Brown-out Reset Voltage Selection bits->Brown-out Reset Voltage (VBOR) set to 2.45V
#pragma config ZCD = OFF    // ZCD Disable bit->ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
#pragma config PPS1WAY = OFF    // PPSLOCK bit One-Way Set Enable bit->PPSLOCK bit can be set and cleared repeatedly (subject to the unlock sequence)
#pragma config STVREN = ON    // Stack Full/Underflow Reset Enable bit->Stack full/underflow will cause Reset
#pragma config DEBUG = OFF    // Debugger Enable bit->Background debugger disabled
#pragma config XINST = OFF    // Extended Instruction Set Enable bit->Extended Instruction Set and Indexed Addressing Mode disabled
// CONFIG3L
#pragma config WDTCPS = WDTCPS_13    // WDT Period selection bits->Divider ratio 1:262144
#pragma config WDTE = OFF  //was SWDTEN    // WDT operating mode->WDT enabled/disabled by SWDTEN bit
// CONFIG3H
#pragma config WDTCWS = WDTCWS_7    // WDT Window Select bits->window always open (100%); software control; keyed access not required
#pragma config WDTCCS = MFINTOSC    // WDT input clock selector->WDT reference clock is the 32kHz MFINTOSC output
// CONFIG4L
#pragma config BBSIZE = BBSIZE_512    // Boot Block Size selection bits->Boot Block size is 512 words
#pragma config BBEN = OFF    // Boot Block enable bit->Boot block disabled
#pragma config SAFEN = OFF    // Storage Area Flash enable bit->SAF disabled
#pragma config WRTAPP = OFF    // Application Block write protection bit->Application Block not write protected
// CONFIG4H
#pragma config WRTB = OFF    // Configuration Register Write Protection bit->Configuration registers (300000-30000Bh) not write-protected
#pragma config WRTC = OFF    // Boot Block Write Protection bit->Boot Block (000000-0007FFh) not write-protected
#pragma config WRTD = OFF    // Data EEPROM Write Protection bit->Data EEPROM not write-protected
#pragma config WRTSAF = OFF    // SAF Write protection bit->SAF not Write Protected
#pragma config LVP = OFF    // Low Voltage Programming Enable bit->HV on MCLR/VPP must be used for programming
// CONFIG5L
#pragma config CP = OFF    // PFM and Data EEPROM Code Protection bit->PFM and Data EEPROM code protection disabled

#define _XTAL_FREQ 64000000 


#ifndef Byte
#define Byte  uint8_t
#endif
#ifndef Word
#define Word  uint16_t
#endif

#define VERSION "2023-0407"

#define  MCU " 18F27K42"
#define POWER_SUPPLY " 3,3V "
#define DIR " C:\\MPLABX_PrOJECTS"
#define COMPILO "XC8 V2.36 C99"
#define PACK "PIC18F-K_DFP 1.6.125"

#define CR 13
#define LF 10
#define TAB 9

#define SQA_Dir TRISAbits.TRISA3
#define SQA LATAbits.LA3

#define Directory "C:\\MPLABX_Projects\\_18F27K42_OWS_DS18B20_2023.X"
#define Project " 18F27K42_OWS_DS18B20_2023"
#define Source  " main.c "
#define Annexe  "  "
#define Others  " DS18B20_barker.h  OneWire_barker.h"
#define Eeprom  " Non"
#define Hardware " Base 18F27K42 o "
#define Config " Internal Fosc 64MHz"

#define With_Test_Presence_Devices 

#include "DS18B20_barker.h"  

  const  char Msg0[]=" PIC18F27K42 + OWS sensor DS18B20 sur  PORT RA0)\r\n";
  const  char Msg1[]=" Directory :"Directory"\r\n";
  const  char Msg2[]=" Project : "Project"\r\n";
  const  char Msg3[]=" Source : "Source"_"VERSION"\r\n";
  const  char Msg4[]=" Config "Config"\r\n";
  const  char Msg5[]=" Eeprom :"Eeprom"\r\n";
  const  char Msg7[]=" ";
  const  char Msg6[]=" RA0=DS18B20 RA4=led  RA3=SQA RC6=TX UART1\r\n";
  const  char Msg8[]=" Autres : "Others"\r\n";
  const  char Msg9[2]=" ";
  const  char * Messages[]={Msg0,Msg1,Msg2,Msg3,Msg4,Msg5,Msg6,Msg7,Msg8,Msg9};
  const  char Blancs[]="  ";
  
 //------------------------
 //PCF8574 pour LCD 2x16 avec A0,A1,A2  au niveau logique 0
  #define LCD_ADDR 0x20  
 //#define LCD_ADDR 0x27  //PCF8574 pour gros LCD 4x20 
 // Hardware :
 // PCF8574 I2C LCD Port Definition
 // P7,P6,P5,P4 = 4 bits Data  -> LCD D7..D4
 // P0=RS ,  P1=RW  not used car RW=0V , P2=Enb  P3=Backlight (BL) not used
 #define LCD_BL 0x08      // P3
 #define LCD_EN 0x04      // P2
 #define LCD_RW 0x02      // P1
 #define LCD_RS 0x01      // P0

#define LCD_CLEAR 0x01
#define LCD_HOME  0x02
#define Display_ON 0x0C
#define Display_OFF 0x08
  
  
  
 // LCD Command
#define LCD_LINE1 0x80      // 0x00 + bit D7=1
#define LCD_LINE2 0xC0      // 0x40 + bit D7=
#define LCD_LINE3 0x94 // 0x14 + bit D7=1
#define LCD_LINE4 0xD4 // 0x54 + bit D7=1
 
  
volatile int NbCarPL=16;
const unsigned char LINE[5] ={0x00,0x80,0xC0,0x94,0xD4};
const char Efface[17]="                ";

const unsigned char LCD_Custom_Chars_Set1[64] =
{
//---------- set #1 ------------------
0x0C,0x12,0x12,0x0C,0x00,0x00,0x00,0x00,  //char #1   Degre
0x06,0x09,0x08,0x1E,0x1E,0x08,0x09,0x06,  //char #2   euro
0x04,0x0A,0x11,0x11,0x0A,0x0A,0x11,0x00,  // char #3  Ohm
0x04,0x0E,0x0E,0x0E,0x1F,0x00,0x04,0x00,   // char #4   bell
0x1C,0x14,0x1B,0x12,0x13,0x12,0x12,0x00,//char #5 initiales PF
0x01,0x03,0x07,0x0F,0x07,0x03,0x01,0x00,  //char #6  A gauche
0x00,0x11,0x0A,0x04,0x06,0x11,0x00,0x00,  //char #7 cancel
0x00,0x01,0x03,0x16,0x1C,0x10,0x00,0x00  //char #8  Checked
};

const unsigned char LCD_Custom_Chars_Set2[64] =
{

//-----Barre verticales   set #2   0  6 colonnes , mais seulement 5 maxi en 5x8 cars  -------
0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,0b10000000,
0b11000000,0b11000000,0b11000000,0b11000000,0b11000000,0b11000000,0b11000000,0b11000000,
0b11100000,0b11100000,0b11100000,0b11100000,0b11100000,0b11100000,0b11100000,0b11100000,
0b11110000,0b11110000,0b11110000,0b11110000,0b11110000,0b11110000,0b11110000,0b11110000,
0b11111000,0b11111000,0b11111000,0b11111000,0b11111000,0b11111000,0b11111000,0b11111000,
0b11111100,0b11111100,0b11111100,0b11111100,0b11111100,0b11111100,0b11111100,0b11111100,
0b11111110,0b11111110,0b11111110,0b11111110,0b11111110,0b11111110,0b11111110,0b11111110,
0b11111111,0b11111111,0b11111111,0b11111111,0b11111111,0b11111111,0b11111111,0b11111111
};

const unsigned char LCD_Custom_Chars_Set3[64] =
{
//------- Set #3  ----------------------
0,0,0,0,27,27,27,27,
27,27,27,27,0,0,0,0,
3,3,3,3,24,24,24,24,
24,24,24,24,3,3,3,3,
0,0,0,31,31,31,31,31,
0,0,31,31,31,31,31,31,
0,31,31,31,31,31,31,31,
31,31,31,31,31,31,31,31
};

const char Special_Name_1[8][7]=
{"DEGREE","EURO  ","OHM   ","PF    ","Droite","Gauche","CANCEL","Check."}; // pour caracteres Speciaux
const char Special_Name_2[8][7]=
{ "Zero  ","un    ","deux  ","trois ","quatre","cinq  ","six   ","Sept  "};// pour Barre Graphe Horizontale

//---------------------------------------------------
  
  
 
#define UART1_RX_BUFFER_SIZE 128  

volatile uint16_t  i;
uint16_t  j,k,l,m,n;
uint16_t  Rep;
uint8_t  c1,c2,c3,cx,cc;

volatile uint8_t Buffer1[UART1_RX_BUFFER_SIZE];
volatile uint16_t Index1,i1;
volatile uint8_t Flag_Buffer1=0;

uint8_t TEXTE [128];
uint8_t * txt;
uint8_t * p0;
const uint8_t * p1;
const uint8_t * p2;
uint8_t * px;
uint8_t CRam1[80];
uint8_t Tampon[32];

float F0,F1,F2;
volatile uint16_t Count0, Count1 ;
volatile uint16_t Cpt0, Cpt1;
volatile uint16_t Flag_Timer1=0;
volatile uint16_t Flag_Timer3=0;
volatile uint16_t Flag_Timer6=0;
volatile signed int  Cpt6;  
volatile signed int  Cpt3;  
bool Status_GIE_Bit;



volatile unsigned char Addr;
//volatile unsigned char Adresse_7bits;
uint8_t I2_Adresses_Devices[8];
unsigned char tmp[4] ;
int Etat;
int Step=0;



unsigned int DS18B20_raw[3];
unsigned char OWS_address[8]= {0,0,0,0,0,0,0,0};
uint8_t temp1[8] = {1,2,3,4,5,6,7,8}; 
char *temp = "000.00 C";

volatile struct Anybits {   // 8 flags
       unsigned  Tmr0_Elapsed:1;     // for timer0 16bits
       unsigned  Tmr1_Elapsed:1;     // for timer1 16bits
       unsigned  Tmr2_Elapsed:1;     // for timer2 8bits
       unsigned  Tmr3_Elapsed:1;     // for timer3 16bits
       unsigned  Tmr4_Elapsed:1;     // for timer4 8bits
       unsigned  Tmr6_Elapsed:1;     // for timer6 8bits
       unsigned  NCO_initialise:1;   // for NCO
       unsigned  libre:1;
 }Drapeaux ;
 
 
const char DegreC[8] ={    // 	176
0b01110000,
0b01010000,
0b01110111,
0b00000000,
0b00000111,
0b00000100,
0b00000100,
0b00000111
};
 

void Interrupt_Init(void);
void Hardware_Init();
void  UART1_Init(void); 
void TMR6_Init_500ms(char N);
void RAZ_UART1_Buffer1(void);
void PrintChar(uint8_t txData);
void Print (char * T1);
void CPrint (const char * T1);
void CRLF1(void);
void strConstRamCpy(uint8_t  *dest, const  uint8_t *source);
void Decompile_byte(unsigned char un);
void __delay_xS(uint8_t N);


void Init_I2C (void);
void I2C1_Write1ByteRegister(i2c1_address_t address, uint8_t reg, uint8_t data);
void I2C_PCF8574_Write(unsigned char  Addr , unsigned char value);
void Init_2_LCD(void);
void LCD_init(void);
void LCD_putcmd(unsigned char d,unsigned char cmdtype);
void LCD_putch(unsigned char d);
void LCD_puts(char *s);
void LCD_putsRom(const char *s);
void LCD_Write_CText(const char *txt) ;
void Erase_Line(int L);
void LCD_At(unsigned char Ligne,unsigned char  col);
void LCD_Write_Text_At(unsigned char Ligne,unsigned char col, char * t1);
void LCD_Write_CText_At(unsigned char Ligne,unsigned char col,const char * t1);
void LCD_Cmd(unsigned char d);
void LCD_Load_Custom_Chars(unsigned char Set,int bavard);


//--------------------------------------------------------------------------------------------


void  Interrupt_Init(void)
{
    INTCON0bits.IPEN = 1;

    bool state = (unsigned char)  INTCON0bits.GIE;
    
    INTCON0bits.GIEH = 0; // dishigh priority interrupts
    INTCON0bits.GIEL = 0; // dis low priority interrupts
    INTCON0bits.IPEN = 1; // Enable interrupt priority
   //  __asm("BCF INTCON0,GIE");
   // GIE = 0;
    IVTLOCK = 0x55;
    IVTLOCK = 0xAA;
    IVTLOCKbits.IVTLOCKED = 0x00; // unlock IVT

    IVTBASEU = 0;
    IVTBASEH = 0;
    IVTBASEL = 8;

    IVTLOCK = 0x55;
    IVTLOCK = 0xAA;
    IVTLOCKbits.IVTLOCKED = 0x01; // lock IVT

    INTCON0bits.GIE= state;

    // Assign peripheral interrupt priority vectors
    IPR3bits.U1RXIP = 1;
    IPR1bits.INT0IP = 1;
    IPR4bits.TMR1IP = 1;
    IPR0bits.IOCIP=1;
    IPR3bits.TMR0IP = 1;
    
    
}

/*
void (*IOCAF0_InterruptHandler)(void);
void (*IOCAF1_InterruptHandler)(void);
void (*IOCAF2_InterruptHandler)(void);
void (*IOCAF3_InterruptHandler)(void);
void (*IOCAF4_InterruptHandler)(void);
void (*IOCAF5_InterruptHandler)(void);
void (*IOCAF6_InterruptHandler)(void);
*/

void __interrupt(irq(IRQ_U1RX),high_priority) UART1_rx_vect_isr(void)
{
   static char cx;
  //  U1TXB='H';  // pour debugging only
  if ( (PIR3bits.U1RXIF==1) && (PIE3bits.U1RXIE ==1) )
  {   
      c1=U1RXB;
   //   U1TXB='*'; // echo
     if ((c1==13) || (i1>UART1_RX_BUFFER_SIZE-1))
     { 
      Flag_Buffer1=1;
      Buffer1[i1]=0;
      Index1=i1;
      i1=0; 
     PIE3bits.U1RXIE =0;
     }
     else
     {
       Buffer1[i1]=c1;
       i1++;
       Index1=i1;
      } 
   } 
}


void __interrupt(irq(IRQ_TMR1),high_priority)TMR1_ISR(void)
{        
     if ( Cpt1 < 1)   // x 100ms at 500Khz
      {
      T1CONbits.TMR1ON=0;// stop comptage  timer1
      PIE4bits.TMR1IE=0;
      Drapeaux.Tmr1_Elapsed=1;
      }
      else
      {
       Cpt1--;
      }
      // 100mS  avec prescaler =1/2  et bascule  1/2
      TMR1H         = 0xCF;  // 53036
      TMR1L         = 0x2C;
       PIR4bits.TMR1IF=0;
  }

void __interrupt(irq(IRQ_TMR6),high_priority) TMR6_ISR(void)
{  
    if (  (TMR6IF==1) && (TMR6IE==1) )
    {
      if (Cpt6>0)  // decomptage maxi 32767
       {
         Cpt6--;
        }
      else
      {
        Drapeaux.Tmr6_Elapsed=1;
        TMR6IE=0;
      } 
     Flag_Timer6=1;
     TMR6IF=0;// clear the TMR6 interrupt flag
     
    }       
 }


void UART1_Init(void)
{
    U1P1L = 0x00;
    U1P1H = 0x00;
    U1P2L = 0x00;
    U1P2H = 0x00;
    U1P3L = 0x00;
    U1P3H = 0x00;
    U1CON0 = 0xB0; // BRGS high speed; MODE Asynchronous 8-bit mode; RXEN enabled; TXEN enabled; ABDEN disabled; 
    // RXBIMD Set RXBKIF on rising RX input; BRKOVR disabled; 
    // WUE disabled; SENDB disabled; ON enabled; 
    U1CON1 = 0x80;    
    // TXPOL not inverted; FLO off; C0EN Checksum Mode 0; RXPOL not inverted;
    //RUNOVF RX input shifter stops all activity; STP Transmit 1Stop bit, receiver verifies first Stop bit; 
    U1CON2 = 0x00;
    U1BRGL = 0x8A; // BRGL 138;   115200 bds
    U1BRGH = 0x00; // BRGH 0; 
    U1FIFO = 0x00; // STPMD in middle of first Stop bit; TXWRE No error; 
    // ABDIF Auto-baud not enabled or not complete; 
    // WUIF WUE not enabled by software; ABDIE disabled; 
    U1UIR = 0x00;
    // ABDOVF Not overflowed; TXCIF 0; RXBKIF No Break detected; 
    // RXFOIF not overflowed; CERIF No Checksum error; 
    U1ERRIR = 0x00;
    // TXCIE disabled; FERIE disabled; TXMTIE disabled; ABDOVE disabled;
    // CERIE disabled; RXFOIE disabled; PERIE disabled; RXBKIE disabled; 
    U1ERRIE = 0x00;
    U1CON0bits.TXEN=1;
    U1CON0bits.RXEN=1;  // enable recpetion
    PIE3bits.U1RXIE = 0;  // disable receive interrupt
}



void strConstRamCpy(uint8_t  *dest, const  uint8_t *source)
 { static uint8_t cc=0;
 
  while (*(dest++) == *(source++) ) 
  { 
  };
// cc= *(source);
//  while ( cc>0 )
//  { cc=*(source);
//    *(dest++) =cc;
//    source++ ;
//  }
//  *(dest) = 0 ;    // terminateur
}


void __delay_xSec(uint8_t N)
{
static int i2;  
for (i2=0;i2<N;i2++)  __delay_ms(1000);
}

void PrintChar( uint8_t txData)
{
    while(0 == PIR3bits.U1TXIF)
    {
        __asm("NOP");
    }
    U1TXB = txData;    // Write the data byte to the USART.
}

 void Print (char * Tc)
 { static uint16_t k;
   k=0;      
   while(*(Tc+k)>0) 
   {
     while(PIR3bits.U1TXIF==0);
     U1TXB=*(Tc+k);
    k++;
    if(k>128)break;
   }
}


  void CPrint (const char *T1)
 { 
   while(*(T1)>0) 
   {
     while(PIR3bits.U1TXIF==0);
     U1TXB=*(T1);
    T1++;
   }
}

void CRLF1(void)
{
   while(PIR3bits.U1TXIF==0);
   U1TXB=13;
    while(PIR3bits.U1TXIF==0);
   U1TXB=10;  
  }   


void RAZ_UART1_Buffer1(void)
{ int i1;   
      if (PIR3bits.U1RXIF==1)   c1=U1RXB ;
      for (i1=0;i1<sizeof(Buffer1)-1;i1++) Buffer1[i1]=0;
      Flag_Buffer1 =0 ;
      U1CON0bits.U1RXEN=1;
      U1ERRIR = 0x00;
      U1ERRIE = 0x00;
      Buffer1[0]=0;
      i1=0;
      Index1=0;
      c1=0;
      PIE3bits.U1IE  = 1;       // int uart1 enabled
      PIE3bits.U1RXIE =1;  
} 


void TMR1_Initialize(void)
{
    //Set the Timer to the options selected in the GUI
    //T1GE disabled; T1GTM disabled; T1GPOL low; T1GGO done; T1GSPM disabled; 
    T1GCON = 0x00;
    T1GATE = 0x00;//GSS T1G_pin; 
    T1CLK = 0x05; //CS MFINTOSC_500kHz; 
    TMR1H = 0x0B;  //TMR1H 11;
    TMR1L = 0xDC; //TMR1L 220; 
    // Clearing IF flag before enabling the interrupt.
    PIR4bits.TMR1IF = 0;
    // Enabling TMR1 interrupt.
    PIE4bits.TMR1IE = 1;
    // CKPS 1:4; NOT_SYNC synchronize; TMR1ON enabled; T1RD16 disabled; 
    T1CON = 0x21;
    // Start the Timer by writing to TMRxON bit
    //T1CONbits.TMR1ON = 1;
}


void TMR6_Init_500ms(char N)
{   
    //  cycle=500 mS modifi -> 1sec !
     PIE9bits.TMR6IE=0 ;
     // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T6CLKCON = 0x05;    // T2CS FOSC/4 ou    0x05 pour 500Khz
    T6HLT = 0x00;
    T6TMR = 0x00;  // TMR4 0; 
    T6CON = 0x70 | 0x07 ; // Prescaler =7=> 1/128 Poscaler=7=>  1/8 =>  ticks= 512  S
    T6PR=244;   // 512 * (244+1) * 128 * 8 => 501760S ->  500mS sec
   Flag_Timer6=0;
   Drapeaux.Tmr6_Elapsed=0;
   TMR6ON = 1;
   TMR6IF=0;
   if(N>0) 
   { 
     Cpt6=N;
      PIE9bits.TMR6IE=1 ;  // TMR6IE_bit=1;
   }
   else
   {
       PIE9bits.TMR6IE=0 ; // TMR6IE_bit=0;
        Cpt6=0;      
   } 
 TMR6IF=0;   
 }


void Hardware_Init(void)
{
    LATA = 0x00;
    LATB = 0x00;
    LATC = 0x00;

    TRISA = 0xFF;
    TRISB = 0xD7;
    TRISC = 0x87;  

    ANSELC = 0x00;
    ANSELB = 0x00;
    ANSELA = 0x00;

    WPUE = 0x00;
    WPUB = 0x3F;
    WPUA = 0xFE;
    WPUC = 0xC0;

   // RxyI2C registers
    RB1I2C = 0x00;
    RB2I2C = 0x00;
    RC3I2C = 0x00;
    RC4I2C = 0x00;

   // ODx registers
    ODCONA = 0x01;  //  0x01 => open drain sur RA0 OWS sensor
    ODCONB = 0x00;
    ODCONC = 0x18;  

   
  //  SLRCONx registers
    SLRCONA = 0xFF;
    SLRCONB = 0xFF;
    SLRCONC = 0xFF;

   
   // INLVLx registers
    INLVLA = 0xFF;
    INLVLB = 0xFF;
    INLVLC = 0xFF;
    INLVLE = 0x08;

    //IOCx registers 
        //interrupt on change PORT A   IOCAF - flag
    IOCAF=0;
    IOCAN=0;
    IOCAP=0;
    //IOCAPbits.IOCAP0 = 1;
    // register default IOC callback functions at runtime; use these methods to register a custom function
   /*
    *  IOCAF0_SetInterruptHandler(IOCAF0_DefaultInterruptHandler);
    IOCAF1_SetInterruptHandler(IOCAF1_DefaultInterruptHandler);
    IOCAF2_SetInterruptHandler(IOCAF2_DefaultInterruptHandler);
    IOCAF3_SetInterruptHandler(IOCAF3_DefaultInterruptHandler);
    IOCAF4_SetInterruptHandler(IOCAF4_DefaultInterruptHandler);
    IOCAF5_SetInterruptHandler(IOCAF5_DefaultInterruptHandler);
    IOCAF6_SetInterruptHandler(IOCAF6_DefaultInterruptHandler);
      
    // Enable IOCI interrupt 
    PIE0bits.IOCIE = 1; 
    */
         
	// I2C1
    I2C1SDAPPS = 0x14;  //RC4->I2C1:SDA1;    
    RC3PPS = 0x21;      //RC3->I2C1:SCL1;    
    RC4PPS = 0x22;      //RC4->I2C1:SDA1; 
    // UART1
    RC6PPS = 0x13;      //RC6->UART1:TX1;    
    U1RXPPS = 0x17;     //RC7->UART1:RX1;    
    I2C1SCLPPS = 0x13;  //RC3->I2C1:SCL1;    
}



void Init_I2C (void)
{   
    I2C1CLK    = 0x03;      // I2C1 Clock Selection Register, MFINTOSC 500kHz ?? 100kHz ??  // doc p. 581
  // 2-04-2023  probleme maxi pCF SLC =100KHz
  //  I2C1CLK    = 0x06;    ///0110	 TMR2 post scaled output	42Khz theorique
  // SQA mesure  18.00+11.76=29.76S -> 33.6Khz ?? why ?
    I2C1CON0   = 0x04;      // I2C1 Control Register 0, Master mode, 7-bit Address  // doc p.577
    I2C1CON1   = 0x80;      // I2C1 Control Register 1, Acknowledge value transmitted after received data 
                            //(when I2CCNT=0) : 1 = NACK     (Not ACKnowledge)  // doc p.579
    I2C1CON2   = 0x00;      // I2C1 Control Register 2, envoi Start non auto, SDAHT 300ns hold time, BFRET 8 i2c clock pulses  // doc p.580
    I2C1CNT = 0;
 //   I2C1CON2bits.FME=1;
    I2C1CON2bits.ABD=0; //  ADB=0; // Transmitted Address data is loaded from the I2CxADB0/1 registers.
     // I2C1ADB1   = 0x3C;      //0x3C adresse bus I2C mini oled SSD1306
    I2C1ADB1   = 0x20 ;   //LCD_ADDR;   // LCD 2x16 avec PCF8754
    I2C1PIRbits.PCIF = 0;  // clear STOP bit flag
  //  I2C1CON2bits.ABD=1;   
    I2C1CON0bits.EN=0;   // i2c1 OFF   
}


void Init_Timer2()
{ 
  T2CON    = 0x00;
 // T2CONbits.CKPS=0b000;
   T2CONbits.CKPS2=0;// div 1/1 
    T2CONbits.CKPS1=0;
     T2CONbits.CKPS0=0;
   // T2CONbits.CKPS=0;     
  T2CONbits.OUTPS=1;  // div 1/2
  T2PR=190;  // 0.0625 * 1 * 2 * 190 =23.75 S  =>  42Khz
  T2TMR=0;
  T2CLK=0x02; //1= FOSC/4    2=FOSC     5=500Khz   
 // REGISTER 22-2: TxRST: TIMER2 EXTERNAL RESET SIGNAL SELECTION REGISTER p337
 // T2RST=0x04;  // CCP1OUT
    T2RST=0x00;  
  PIE4bits.TMR2IE=0;
  PIR4bits.TMR2IF=0;
  T2CONbits.T2ON=1;
}


 bool I2C1_DevicePresent(int16_t Address)
{
I2C1PIRbits.PCIF = 0;                // clear STOP bit flag
I2C1ADB1 = Address;                  // load slave Addr and R/W
I2C1CNT = 0;
I2C1CON0bits.S = 1;                  // send START + Addr
while (I2C1PIRbits.PCIF == 0);       // wait for STOP
return !I2C1CON1bits.ACKSTAT;
}

void Test_Presence_I2C_devices()
{
 int i,j;
 txt=&TEXTE[0]; *(txt)=0;
// I2C1CON2 = 0x00;  // <- OK
I2C1CON2bits.ABD=0; //  ADB=0; // Transmitted Address data is loaded from the I2CxADB0/1 registers.
I2C1CON0bits.EN = 1;
for (i=2; i<252; i=i+2)
{
if (I2C1_DevicePresent(i))
{
k=i>>1;
sprintf(txt," @ decimal #%4u  soit @Device 7bits = 0X%02X ",i,k);
switch (k)
{
case 0x20:// de base + 7 autres
case 0x21:
case 0x22:
case 0x23:
    //...etc 
case 0x27:
   txt=strcat(txt,(const char *)" PCF8754 for LCD 2x16cars"); 
   break;
case 0x28:
case 0x29:
case 0x2A:
// case etc ..
strConstRamCpy(CRam1,"PCF8574 8bits IO" );  
j=strlen(txt);
strcpy(txt+j,CRam1);
//txt=strcat(txt,CRam1);// de base + 7 autres
break;
case 0x37:
txt=strcat(txt,(const char *)"LCD 2119 2x16cars"); // de base + 7 autres
break;
case 0x38:
case 0x39:
case 0x3A:
case 0x3B:
// case etc ..
strcat(txt,"PCF8574A 8bits I/O"); // de base + 7 autres
break;
case 0x3C:
strcat(txt,"OLED SSD1306");
break;
case 0x57:
strcat(txt,"EEPROM 512bytes");
break;
case 0x68:
strcat(txt,"RTC DS3231");
break;
default:
strcat(txt,"non recens");
break;
}
Print(txt);CRLF1();
}
}
I2C1CON0bits.EN = 0;
//I2C1CON2 = 0x18;
I2C1CON2bits.ABD=1;
CRLF1();
}


void I2C_PCF8574_Write(unsigned char Adrp,unsigned char value)
{ 
 tmp[0]=value;
 tmp[1]=0;
 I2C1_WriteNBytes(Adrp, tmp, 1); 
  __delay_us(30);
} 

//------------------------------------------------------------- 


void Erase_Line(int L)
{
switch(L)
 {
 case 1:
  {
   LCD_Cmd(LCD_LINE1);   //LCD_putcmd(LCD_LINE1,1);   
    LCD_putsRom(Efface);      
   LCD_Cmd(LCD_LINE1); //LCD_putcmd(LCD_LINE1,1);            
   break;
  }
  case 2:
  {
   LCD_Cmd(LCD_LINE2); //LCD_putcmd(LCD_LINE2,1);   
   LCD_putsRom(Efface);     
   LCD_Cmd(LCD_LINE2);//LCD_putcmd(LCD_LINE2,1); 
    break;
  } 
  case 3:
  {
   LCD_Cmd(LCD_LINE3); //LCD_putcmd(LCD_LINE2,1);   
   LCD_putsRom("                    ");
   LCD_Cmd(LCD_LINE3);//LCD_putcmd(LCD_LINE2,1); 
    break;
  } 

  case 4:
  {
   LCD_Cmd(LCD_LINE4); //LCD_putcmd(LCD_LINE2,1);   
    LCD_putsRom("                    ");   
   LCD_Cmd(LCD_LINE4);//LCD_putcmd(LCD_LINE2,1); 
    break;
  } 

  }
 __delay_ms(10);
}

 //  LCD 2x16 + PCF8753
 // PCA8574 I2C LCD Routine
 // LCD Data PCA8574: P0,P1,P2,P3     Poids faibles !
 // LCD Control: P7: Back Light not used fix avec R pull up., P4=E-Enable, P5=not Used RW au Grnd , P6= RS



void LCD_putcmd(unsigned char d,unsigned char cmdtype)
{
  unsigned char lcddata;
 // avec RS=0
    // Put the Upper 4 bits data  on P7..P4
  lcddata =  (d & 0xF0) | LCD_EN | LCD_BL ;
   I2C_PCF8574_Write(Addr,lcddata );
  __delay_us(15);
  // Write Enable Pulse E: Hi -> Lo
   lcddata =  (d & 0xF0)| LCD_BL  ;
   I2C_PCF8574_Write(Addr,lcddata);
  __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
  // cmdtype = 0; One cycle write, cmdtype = 1; Two cycle writes
  if (cmdtype==1)
  {
    // Put the Lower 4 bits data   on P7 ..P4
    lcddata = ((d & 0x0F) <<4) | LCD_EN | LCD_BL ;
     I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);    // __delay 2us for 16 MHz Internal Clock
    // Write Enable Pulse E: Hi -> Lo
     lcddata = ((d & 0x0F)<<4) | LCD_BL ;
     I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);    // __delay 1us for 16 MHz Internal Clock
  }
}



void LCD_putch(unsigned char d)
{
  unsigned char lcddata;
   // Put the Upper 4 bits data     on P0 ..P3
  // avec RS=1
  lcddata =( (d & 0xF0) >>4 )| LCD_RS;
  I2C_PCF8574_Write(Addr,lcddata | LCD_EN);
  __delay_us(15);      // __delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
  lcddata = (  ( (d & 0xF0) >>4 ) |  LCD_RS ) & (~LCD_EN) ;
  I2C_PCF8574_Write(Addr,lcddata);
  __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
  // Put the Lower 4 bit data
  lcddata = (d & 0x0F) | LCD_RS | LCD_EN;
  I2C_PCF8574_Write(Addr,lcddata);
  __delay_us(15);      // __delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
  lcddata = ( (d & 0x0F) | LCD_RS ) & (~LCD_EN);
  I2C_PCF8574_Write(Addr,lcddata);
  __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
}


void LCD_Cmd(unsigned char d)
{
  unsigned char lcdcmd;
 // avec RS=0
    // Put the Upper 4 bits data  on P7..P4
    lcdcmd =   (d & 0xF0)  | LCD_EN | LCD_BL;
    I2C_PCF8574_Write(Addr,lcdcmd );
  __delay_us(15);
  // Write Enable Pulse E: Hi -> Lo
  // lcdcmd =  ( (d & 0xF0) >> 4  | LCD_RW  )  & (~ LCD_EN);
     lcdcmd =   (d & 0xF0) | LCD_BL ;
     I2C_PCF8574_Write(Addr,lcdcmd);
    __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
  // Put the Lower 4 bits data   on P7....P4
     lcdcmd = ((d & 0x0F)<<4)| LCD_EN | LCD_BL;
     I2C_PCF8574_Write(Addr,lcdcmd);
      __delay_us(15);    // __delay 2us for 16 MHz Internal Clock
    // Write Enable Pulse E: Hi -> Lo
     lcdcmd = (d & 0x0F)<<4 | LCD_BL ;
     I2C_PCF8574_Write(Addr,lcdcmd);
      __delay_us(15);    // __delay 1us for 16 MHz Internal Clock
}


void LCD_Data(unsigned char d)
{
  static  unsigned char lcddata;
   // Put the Upper 4 bits data     on P0 ..P3
  // avec RS=1
  lcddata=  ((d & 0xF0) )| LCD_RS | LCD_EN | LCD_BL;
  I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);      // __delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
 // lcddata =  ( ( (d & 0xF0) >>4 ) |  LCD_RS | LCD_RW  )  & (~LCD_EN);
  lcddata =   (d & 0xF0)  | LCD_RS | LCD_BL ;
  I2C_PCF8574_Write(Addr,lcddata);
   __delay_us(15);      // __delay 1us for 16 MHz Internal Clock
  // Put the Lower 4 bit data
 // lcddata = (d & 0x0F) | LCD_RS | LCD_EN | LCD_RW;
  lcddata =  ( (d & 0x0F) <<4 ) | LCD_RS | LCD_EN | LCD_BL;
  I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);      // __delay 2us for 16 MHz Internal Clock
  // Write Enable Pulse E: Hi -> Lo
   lcddata =  ( (d & 0x0F) <<4 ) | LCD_RS | LCD_BL ;
  I2C_PCF8574_Write(Addr,lcddata);
    __delay_us(15);     // __delay 1us for 16 MHz Internal Clock
}



void LCD_init(void)  // OK !
{
  
  // Wait for more than 15 ms after VCC rises to 4.5 V
  PrintChar('.');
  __delay_ms(200);
  // Send Command 0x30
  PrintChar('1');
  LCD_putcmd(0x30,0);
  // Wait for more than 4.1 ms
  __delay_ms(5);
   PrintChar('2');
  // Send Command 0x30
  LCD_putcmd(0x30,0);
  __delay_ms(1);
  // Send Command 0x30
     PrintChar('3');
  LCD_putcmd(0x30,0);
    __delay_ms(1);
  // Function set: Set interface to be 4 bits long (only 1 cycle write).
      PrintChar('4');
 LCD_putcmd(0x25,0); 
    __delay_ms(1);
  // Function set: DL=0;Interface is 4 bits, N=1; 2 Lines, F=0; 5x8 dots font)
       PrintChar('5');
  LCD_putcmd(0x28,1);
     __delay_ms(1);
  // Display Off: D=0; Display off, C=0; Cursor Off, B=0; Blinking Off
       PrintChar('6');
  LCD_putcmd(0x08,1);
    __delay_ms(1);
  // Display Clear
       PrintChar('7');
  LCD_putcmd(LCD_CLEAR,1);
    __delay_ms(25);
  // Entry Mode Set: I/D=1; Increament, S=0; No shift
       PrintChar('8');
  LCD_putcmd(0x06,1);
      __delay_ms(1);
  // Display On, Cursor Off
       PrintChar('9');
  LCD_putcmd(0x0C,1);
    __delay_ms(1);
        PrintChar('A');
}



void LCD_putsRom(const char *s1)
{  uint16_t i;
    i=0;
   while((*s1 != 0)&& (i<NbCarPL))
  {
    LCD_Data(*s1); //     LCD_putch(*s1);
    s1++;i++;
     __delay_ms(1);
  }
}

void LCD_puts( char *s)
{  int i;
   i=0;
  while((*(s+i) != 0) && (i<NbCarPL))
  {
    LCD_Data(*(s+i));  // LCD_putch(*(s+i));
    i++;
  }
}
  

void LCD_Write_CText(const char *txt)
{  static int i1;
   i1=0;
   while ((*txt) && (i1<NbCarPL))
   i1++;
   LCD_Data(*(txt++)); //LCD_putch(*(txt++));
    __delay_ms(1);
 }


void LCD_Chr_At(int Ligne,int col,char c1)
{
  // Addr=LCD_ADDR;
  if(col>NbCarPL) col=NbCarPL;
  if (Ligne==1) LCD_Cmd(LCD_LINE1+col-1);  
  if (Ligne==2) LCD_Cmd(LCD_LINE2+col-1); 
   __delay_ms(25);
    LCD_Data(c1); //  LCD_putch(c1);
   __delay_ms(25);
 }


void LCD_At(unsigned char Ligne,unsigned char col)
{
  //Addr=LCD_ADDR;
  if(col>NbCarPL) col=NbCarPL-1;
  if (Ligne==1) LCD_Cmd(LCD_LINE1+col-1);  
  if (Ligne==2) LCD_Cmd(LCD_LINE2+col-1);  
  __delay_ms(20);
}

void LCD_Write_Text_At(unsigned char Ligne,unsigned char Col, char * t1)
{  LCD_At(Ligne,Col);
   LCD_puts(t1);
}


void LCD_Write_CText_At(unsigned char Ligne,unsigned char col, const char * t1)
{ char Table1[17];
 int k;
  k=strlen(t1);
  strConstRamCpy(Table1,t1);
  *(Table1+16)=0;
  if(col>=NbCarPL-k) col=1; 
  if (Ligne==1) LCD_Cmd(LCD_LINE1+col-1);
  if (Ligne==2) LCD_Cmd(LCD_LINE2+col-1);
  __delay_ms(20);
  LCD_puts(Table1);
}


void Decompile_byte(unsigned char un)
{
 unsigned char masque;
 masque = 0x80;
 while (masque > 0u )
 {
   if ((un & masque)>0)
      PrintChar('*');  //   UART1_Write(49u);  //  '1'
    else
       PrintChar(' ');  // UART1_Write(48u);  //  '0'
    masque =masque >>1;
  }
}

//   affiche les pixels de chaque jeux de caracteres sur le terminal UART
void LCD_Load_Custom_Chars(unsigned char Set, int Cause)
{
    
 unsigned int  i, j;
 unsigned int  k;
 // Set address counter pointing to CGRAM address 0.
 LCD_Cmd(0x40);
  __delay_ms(150);
 // Load custom lcd character data into CGRAM.
 // It can only hold a maximum of 8 custom characters.
 k=0;
 if (Set==1)  CPrint(Special_Name_1[k]);
 if (Set==2)   CPrint(Special_Name_2[k]);
  if (Set==3)   CPrint(Special_Name_2[k]);// 2 car meme nomage
 CRLF1();
 k=1;
 for(i = 0; i <64; i++)
  {
    if ( (i>0) & (( i%8) ==0))
    {
    if (Set==1)  CPrint(Special_Name_1[k]);
    if (Set==2)   CPrint(Special_Name_2[k]);
    if (Set==3)   CPrint(Special_Name_2[k]); // meme nomage !
     k++;
     CRLF1();
    }
    switch (Set)
    {
        case 1:
         LCD_Data(LCD_Custom_Chars_Set1[i]);
         if (Cause==1)  Decompile_byte( LCD_Custom_Chars_Set1[i]);
            break;
       case 2:
         LCD_Data(LCD_Custom_Chars_Set2[i]);
         if (Cause==1)Decompile_byte(  LCD_Custom_Chars_Set2[i]);
            break;
       case 3:
         LCD_Data(LCD_Custom_Chars_Set3[i]);
         if (Cause==1)Decompile_byte(  LCD_Custom_Chars_Set3[i]);
            break;
      default:
          LCD_Data(LCD_Custom_Chars_Set1[i]);
         if (Cause==1)Decompile_byte( LCD_Custom_Chars_Set1[i]); 
         break; 
    }
    __delay_ms(25);
   if (Cause==1) CRLF1();
  }
 // Set address counter pointing back to the DDRAM.
 LCD_Cmd(0x80);
  __delay_ms(150);
}

//---------------------------------------------------


void main(void)
{

    OSCCON1 = 0x60;
    OSCCON3 = 0x00;
    OSCEN = 0x00;
    OSCFRQ = 0x08;  // 64Mhz
    OSCTUNE = 0x00;
    __delay_ms(1500);
    Hardware_Init();
    SQA_Dir=0;
    SQA=0;
    txt=&TEXTE[0];
    for (i=0;i<sizeof(Buffer1);i++) Buffer1[i]=0;
    for (i=0;i<sizeof(CRam1);i++) CRam1[i]=0;
      
    UART1_Init();
    __delay_ms(500);
    CRLF1();
    CPrint(" Presentation :");
    for (i=0;i<9;i++)
    {
     CPrint(Messages[i]);   
    }
    CRLF1();
    
     CPrint(" Init_I2C();\r\n");
     Init_I2C(); // 500Khz / 5 = 100Khz
 
    
#ifdef With_Test_Presence_Devices
    CPrint(" Test presence devices sur Bus I2C1\r\n");
    Test_Presence_I2C_devices();
    CRLF1();
#endif    
    
    Addr=LCD_ADDR; 
    NbCarPL=16;
    sprintf(CRam1," adresse LCD= %d  %02x \r\n",Addr,Addr);
    Print(CRam1);
    I2C1CON2bits.ABD=1;
    CRLF1();
    
    CPrint(" \r\nInit 2 LCD I2C via PCF8754\r\n");
    I2C1CON0bits.EN=1;   
    LCD_init(); 
    LCD_Cmd(LCD_CLEAR);// 0x01
     __delay_xSec(1);
    LCD_Cmd(LCD_HOME); //  0x02
    CPrint(" Affichage sur LCD 2x16 \r\n");
    
    LCD_putsRom("ABCDEFGHIJKLMNOP");
    Erase_Line(2);  __delay_ms(100);
    LCD_putsRom("1234567890123456");
    __delay_xSec(2);
    
           
    CPrint(" \r\n Config et Init One Wire\r\n");
    onewireInit();
    CPrint(" Test prsence d'un capteur O.W.S. sur RA0\r\n");
    broadcastConvert() ;
    CPrint("  Sensor Datas :\r\n");
    printSingleAddress();

    i=0;
   
    LCD_Load_Custom_Chars(1,0);  // Set#1  mode silence
    LCD_Cmd(LCD_CLEAR);
    LCD_Cmd(LCD_HOME); //  0x02
    //           1234567890123456
    LCD_putsRom("T.EXTER.        ");        
     
     
     CPrint(" Mesure DS18B20 #1 : \r\n");
    do
    {
       broadcastConvert(); 
       SQA=1;
       F1= getTemperature( OWS_address);
       SQA=0;
       sprintf(txt," %05d  temp = %5.3f %c C \r\n",i,F1,176);
       Print(txt);
      
       sprintf(CRam1,"%3.2f C",F1);
       LCD_Write_Text_At(1,10,CRam1);
       LCD_Chr_At(1,15,0);   // degre C
       __asm("btg LATB,5");
        i++;
        __delay_xSec(2);
    }
     while (1);  
}

/*
 Presentation : PIC18F27K42 + OWS sensor DS18B20 sur  PORT RA0)
 Directory :C:\MPLABX_Projects\_18F27K42_OWS_DS18B20_2023.X
 Project :  18F27K42_OWS_DS18B20_2023
 Source :  main.c _2023-0406
 Config  Internal Fosc 64MHz
 Eeprom : Non
 RA0=DS18B20 RA4=led  RA3=SQA RC6=TX UART1
  Autres :  DS18B20_barker.h  OneWire_barker.h

 
 Config et Init One Wire
 Test prsence d'un capteur O.W.S. sur RA0
  Sensor Datas :
 Code Famille : DS18B20
 OWS ID# = AA.42.65.26.13.02.B6.
 CRC = B6 

 Mesure DS18B20 #1 :  00000  temp = 22.687  C 
 00001  temp = 22.625  C 
 00002  temp = 22.500  C 
 00003  temp = 22.437  C 
 00004  temp = 22.312  C 
 00005  temp = 22.250  C 
 00006  temp = 22.187  C 
 00007  temp = 22.062  C 

  */